@@ -1,6 +1,6 @@
 /*
 	conffile.cc	Configuration File Functions
-	Copyright (c) 2000, 2001 Kriang Lerdsuwanakij
+	Copyright (c) 2000, 2001, 2002 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -21,6 +21,7 @@
 #include "conffile.h"
 
 #include CXX__HEADER_cstdio
+#include CXX__HEADER_cctype
 #if HAVE_UNISTD_H
 # include <unistd.h>
 #endif
@@ -207,8 +208,7 @@ bool	IsYesNo(const char *cmd, char *buf, size_t from, size_t len)
 // Return >=0 - OK, -1 Error
 unsigned	ReadUnsigned(const char *cmd, char *buf, size_t from, size_t len)
 {
-			// FIXME: Use size_t here
-	int	value = 0;
+	unsigned	value = 0;
 
 	if (from >= len) {
 		NumberRequired(cmd, buf);
@@ -218,8 +218,10 @@ unsigned	ReadUnsigned(const char *cmd, char *buf, size_t from, size_t len)
 		if (!isdigit(buf[from]))
 			NumberRequired(cmd, buf);
 
+		unsigned old_value = value;
+
 		value = value*10 + buf[from]-'0';
-		if (value < 0)		// Overflow
+		if (value < old_value)		// Overflow
 			PrintErrorMessage(_("number out of range"), cmd, buf);
 
 		from++;
@@ -256,7 +258,7 @@ size_t	SeekEndQuoteOrThrow(const char *cmd, const char *buf, size_t from, size_t
 class	IsPatternMatch {
 		string	pattern;
 	public:
-		IsPatternMatch(string &pattern_) : pattern(pattern_) {}
+		IsPatternMatch(const string &pattern_) : pattern(pattern_) {}
 		int	operator ()(DirItem &str)
 		{
 					// fnmatch returns 0 if strings match
@@ -269,31 +271,83 @@ class	IsPatternMatch {
 *************************************************************************/
 
 struct	ConfigValue {
-	int	*cfgIntPtr;
-	bool	*cfgBoolPtr;
-	DirList	**cfgStringListPtr;
-	AttrConfig *cfgAttrPtr;
+	unsigned KcdConfig::*cfgUIntPtr;
+	COLOR_TYPE KcdConfig::*cfgColorPtr;
+	bool	KcdConfig::*cfgBoolPtr;
+	DirList	* KcdConfig::*cfgStringListPtr;
+	AttrConfig KcdConfig::*cfgAttrPtr;
 };
 
+/*************************************************************************
+	Profiles.
+*************************************************************************/
+
+string	currentProfile;		// Profile from invoked command line
+string	configProfile;		// Current processing profile in
+				// configuration file
+
+bool	IsProfileValid(const string &s, bool allowGlobal)
+{
+	if (s.size() == 0)
+		return false;
+	if (allowGlobal && s == "*")
+		return true;
+
+	if (s[0] != '_' && !isalpha(s[0]))
+		return false;
+	for (size_t i = 1; i < s.size(); ++i) {
+		if (s[i] != '_' && !isalnum(s[i]))
+			return false;
+	}
+	return true;
+}
+
+void	SetCurrentProfile(const string &s)
+{
+	currentProfile = s;
+}
+
+string	GetCurrentProfile()
+{
+	return currentProfile;
+}
+
+void	SetConfigProfile(const string &s)
+{
+	configProfile = s;
+}
+
+bool	IsConfigProfileGlobal()
+{
+	if (configProfile == "*")
+		return true;
+	return false;
+}
+
 /*************************************************************************
 	Command processing/printing functions - Yes/No.
 *************************************************************************/
 
+void	SetYesNo(const ConfigValue &cfg, bool b)
+{
+	allConfig[configProfile].*cfg.cfgBoolPtr = b;
+}
+
 void	ConfigYesNo(const char *cmd, char *buf, size_t from, size_t len,
 		    const ConfigValue &cfg)
 {
 	from = SeekPastEqualSignOrThrow(cmd, buf, from, len);
 
-	bool	yn = IsYesNo(cmd, buf, from, len);
+	bool	b = IsYesNo(cmd, buf, from, len);
 			// Note: error has already been reported by IsYesNo
 
-	*cfg.cfgBoolPtr = yn;
+	SetYesNo(cfg, b);
 }
 
 void	PrintYesNo(const char *cmd, const ConfigValue &cfg)
 {
 	cout << cmd << ": ";
-	if (*cfg.cfgBoolPtr)
+	if (allConfig[configProfile].*cfg.cfgBoolPtr)
 		cout << _("Yes\n");
 	else
 		cout << _("No\n");
@@ -303,26 +357,60 @@ void	PrintYesNo(const char *cmd, const ConfigValue &cfg)
 	Command processing/printing functions - Unsigned integer.
 *************************************************************************/
 
+void	SetUnsigned(const ConfigValue &cfg, unsigned u)
+{
+	allConfig[configProfile].*cfg.cfgUIntPtr = u;
+}
+
 void	ConfigUnsigned(const char *cmd, char *buf, size_t from, size_t len,
 		       const ConfigValue &cfg)
 {
 	from = SeekPastEqualSignOrThrow(cmd, buf, from, len);
 
-	int	thres = ReadUnsigned(cmd, buf, from, len);
+	unsigned u = ReadUnsigned(cmd, buf, from, len);
 			// Note: error has already been reported
 
-	*cfg.cfgIntPtr = thres;
+	SetUnsigned(cfg, u);
 }
 
 void	PrintUnsigned(const char *cmd, const ConfigValue &cfg)
 {
-	cout << cmd << ": " << *cfg.cfgIntPtr << '\n';
+	cout << cmd << ": " << allConfig[configProfile].*cfg.cfgUIntPtr << '\n';
 }
 
 /*************************************************************************
 	Command processing/printing functions - String list.
 *************************************************************************/
 
+void	SetStringList(const ConfigValue &cfg, const string &s)
+{
+						// Reference counted
+	DirItem	str(new DirConfig(s, 
+				  allConfig[configProfile].cfgGlobDot, 
+				  allConfig[configProfile].cfgGlobPath));
+
+	DirList::iterator iter;
+	for (iter = (allConfig[configProfile].*cfg.cfgStringListPtr)->begin(); 
+	     iter != (allConfig[configProfile].*cfg.cfgStringListPtr)->end(); ++iter) {
+
+						// Same directory name
+		if ((*iter)->dir == str->dir) {
+			if (**iter != *str) {
+						// Different glob options
+						// Use new options
+				(allConfig[configProfile].*cfg.cfgStringListPtr)->erase(iter);
+				(allConfig[configProfile].*cfg.cfgStringListPtr)->push_back(str);
+			}
+
+						// Silently ignore
+						// duplicate entry
+			return;
+		}
+	} 
+						// New entry
+	(allConfig[configProfile].*cfg.cfgStringListPtr)->push_back(str);
+}
+
 void	ConfigStringList(const char *cmd, char *buf, size_t from, size_t len, 
 			 const ConfigValue &cfg)
 {
@@ -343,7 +431,7 @@ void	ConfigStringList(const char *cmd, char *buf, size_t from, size_t len,
 		BlankEntry(cmd, buf);
 
 					// Attempt to skip "/" ?
-	if (cfg.cfgStringListPtr == &kcdConfig.cfgSkipDir
+	if (cfg.cfgStringListPtr == &KcdConfig::cfgSkipDir
 	    && strncmp("/", buf+from, end_quote-from) == 0) {
 		PrintErrorMessage(_("cannot skip root directory"), cmd, buf);
 	}
@@ -362,31 +450,7 @@ void	ConfigStringList(const char *cmd, char *buf, size_t from, size_t len,
 		PrintErrorMessage(e.what(), cmd, buf);
 	}
 
-						// Reference counted
-	DirItem	str(new DirConfig(expanded_dir, 
-				  kcdConfig.cfgGlobDot, 
-				  kcdConfig.cfgGlobPath));
-
-	DirList::iterator iter;
-	for (iter = (*cfg.cfgStringListPtr)->begin(); 
-	     iter != (*cfg.cfgStringListPtr)->end(); ++iter) {
-
-						// Same directory name
-		if ((*iter)->dir == str->dir) {
-			if (**iter != *str) {
-						// Different glob options
-						// Use new options
-				(*cfg.cfgStringListPtr)->erase(iter);
-				(*cfg.cfgStringListPtr)->push_back(str);
-			}
-
-						// Silently ignore
-						// duplicate entry
-			return;
-		}
-	} 
-						// New entry
-	(*cfg.cfgStringListPtr)->push_back(str);
+	SetStringList(cfg, expanded_dir);
 }
 
 void	PrintStringList(const char *cmd, const ConfigValue &cfg)
@@ -396,8 +460,8 @@ void	PrintStringList(const char *cmd, const ConfigValue &cfg)
 	bool	indent = false;
 	size_t	num_indent = strlen(cmd)+2;
 
-	for (DirList::iterator iter = (*cfg.cfgStringListPtr)->begin(); 
-	     iter != (*cfg.cfgStringListPtr)->end(); ++iter) {
+	for (DirList::iterator iter = (allConfig[configProfile].*cfg.cfgStringListPtr)->begin(); 
+	     iter != (allConfig[configProfile].*cfg.cfgStringListPtr)->end(); ++iter) {
 		if (indent) {
 			for (size_t i = 0; i < num_indent; ++i)
 				cout << ' ';
@@ -425,6 +489,16 @@ void	PrintStringList(const char *cmd, const ConfigValue &cfg)
 	Command processing/printing functions - Clear string list.
 *************************************************************************/
 
+void	SetClearStringList(const ConfigValue &cfg)
+{
+	(allConfig[configProfile].*cfg.cfgStringListPtr)->clear();
+}
+
+void	SetClearStringList(const ConfigValue &cfg, const string &s)
+{
+	(allConfig[configProfile].*cfg.cfgStringListPtr)->remove_if(IsPatternMatch(s));
+}
+
 void	ConfigClearStringList(const char *cmd, char *buf, size_t from, size_t len, 
 			      const ConfigValue &cfg)
 {
@@ -462,7 +536,7 @@ void	ConfigClearStringList(const char *cmd, char *buf, size_t from, size_t len,
 			PrintErrorMessage(e.what(), cmd, buf);
 		}
 
-		(*cfg.cfgStringListPtr)->remove_if(IsPatternMatch(expanded_dir));
+		SetClearStringList(cfg, expanded_dir);
 	}
 	else {		
 					// Clear everything
@@ -470,7 +544,7 @@ void	ConfigClearStringList(const char *cmd, char *buf, size_t from, size_t len,
 		if (!IsEmpty(buf, from+3, len))
 			ExtraChar(cmd, buf);
 
-		(*cfg.cfgStringListPtr)->clear();
+		SetClearStringList(cfg);
 	}
 }
 
@@ -506,6 +580,18 @@ AttrColorMap colorMap[NUM_COLOR] = {
 	{ "default", DEFAULT_COLOR }
 };
 
+/*************************************************************************
+	Command processing/printing functions - Color attributes
+*************************************************************************/
+
+void	SetColorAttr(const ConfigValue &cfg, ATTR_TYPE colorAttr,
+		     COLOR_TYPE colorForeground, COLOR_TYPE colorBackground)
+{
+	(allConfig[configProfile].*cfg.cfgAttrPtr).colorAttr = colorAttr;
+	(allConfig[configProfile].*cfg.cfgAttrPtr).colorForeground = colorForeground;
+	(allConfig[configProfile].*cfg.cfgAttrPtr).colorBackground = colorBackground;
+}
+
 void	ConfigColorAttr(const char *cmd, char *buf, size_t from, size_t len, 
 			const ConfigValue &cfg)
 {
@@ -514,8 +600,8 @@ void	ConfigColorAttr(const char *cmd, char *buf, size_t from, size_t len,
 	bool first_attr = true;
 	bool found_normal = false;
 	ATTR_TYPE colorAttr = A_NORMAL;
-	int colorForeground = COLOR_WHITE;
-	int colorBackground = COLOR_BLACK;
+	COLOR_TYPE colorForeground = COLOR_WHITE;
+	COLOR_TYPE colorBackground = COLOR_BLACK;
 	int i, tok_end;
 	for ( ; ; ) {
 		tok_end = SeekConfigTokenEnd(buf, from, len);
@@ -597,9 +683,44 @@ void	ConfigColorAttr(const char *cmd, char *buf, size_t from, size_t len,
 	if (from != len)
 		ExtraChar(cmd, buf);
 
-	cfg.cfgAttrPtr->colorAttr = colorAttr;
-	cfg.cfgAttrPtr->colorForeground = colorForeground;
-	cfg.cfgAttrPtr->colorBackground = colorBackground;
+	SetColorAttr(cfg, colorAttr, colorForeground, colorBackground);
+}
+
+void	PrintColorAttr(const char *cmd, const ConfigValue &cfg)
+{
+	cout << cmd << ": ";
+	if ((allConfig[configProfile].*cfg.cfgAttrPtr).colorAttr) {
+		for (int i = 1; i < NUM_ATTR; ++i) {
+			if ((allConfig[configProfile].*cfg.cfgAttrPtr).colorAttr & attrMap[i].value)
+				cout << attrMap[i].name << ' ';
+		}
+	}
+
+	for (int i = 0; i < NUM_COLOR; ++i) {
+		if ((allConfig[configProfile].*cfg.cfgAttrPtr).colorForeground == colorMap[i].value) {
+			cout << colorMap[i].name << ' ';
+			break;
+		}
+	}
+
+	cout << "on ";
+
+	for (int i = 0; i < NUM_COLOR; ++i) {
+		if ((allConfig[configProfile].*cfg.cfgAttrPtr).colorBackground == colorMap[i].value) {
+			cout << colorMap[i].name;
+			break;
+		}
+	}
+	cout << '\n';
+}
+
+/*************************************************************************
+	Command processing/printing functions - Color
+*************************************************************************/
+
+void	SetColor(const ConfigValue &cfg, COLOR_TYPE c)
+{
+	allConfig[configProfile].*cfg.cfgColorPtr = c;
 }
 
 void	ConfigColor(const char *cmd, char *buf, size_t from, size_t len, 
@@ -607,7 +728,7 @@ void	ConfigColor(const char *cmd, char *buf, size_t from, size_t len,
 {
 	from = SeekPastEqualSignOrThrow(cmd, buf, from, len);
 
-	int color = COLOR_BLACK;		// Avoid compiler warning
+	COLOR_TYPE color = COLOR_BLACK;		// Avoid compiler warning
 	int i, tok_end;
 
 	tok_end = SeekConfigTokenEnd(buf, from, len);
@@ -629,7 +750,28 @@ void	ConfigColor(const char *cmd, char *buf, size_t from, size_t len,
 	if (from != len)
 		ExtraChar(cmd, buf);
 
-	*cfg.cfgIntPtr = color;
+	SetColor(cfg, color);
+}
+
+void	PrintColor(const char *cmd, const ConfigValue &cfg)
+{
+	cout << cmd << ": ";
+	for (size_t i = 0; i < NUM_COLOR; ++i) {
+		if (allConfig[configProfile].*cfg.cfgColorPtr == colorMap[i].value) {
+			cout << colorMap[i].name << ' ';
+			break;
+		}
+	}
+	cout << '\n';
+}
+
+/*************************************************************************
+	Command processing/printing functions - Black & white attributes
+*************************************************************************/
+
+void	SetBWAttr(const ConfigValue &cfg, ATTR_TYPE a)
+{
+	(allConfig[configProfile].*cfg.cfgAttrPtr).bwAttr = a;
 }
 
 void	ConfigBWAttr(const char *cmd, char *buf, size_t from, size_t len, 
@@ -669,7 +811,7 @@ void	ConfigBWAttr(const char *cmd, char *buf, size_t from, size_t len,
 		from = tok_end;
 		from = SeekNonSpace(buf, from, len);
 		if (from == len) {		// No more attribute
-			cfg.cfgAttrPtr->bwAttr = bwAttr;
+			SetBWAttr(cfg, bwAttr);
 			return;
 		}
 
@@ -677,52 +819,12 @@ void	ConfigBWAttr(const char *cmd, char *buf, size_t from, size_t len,
 	}
 }
 
-void	PrintColorAttr(const char *cmd, const ConfigValue &cfg)
-{
-	cout << cmd << ": ";
-	if (cfg.cfgAttrPtr->colorAttr) {
-		for (int i = 1; i < NUM_ATTR; ++i) {
-			if (cfg.cfgAttrPtr->colorAttr & attrMap[i].value)
-				cout << attrMap[i].name << ' ';
-		}
-	}
-
-	for (int i = 0; i < NUM_COLOR; ++i) {
-		if (cfg.cfgAttrPtr->colorForeground == colorMap[i].value) {
-			cout << colorMap[i].name << ' ';
-			break;
-		}
-	}
-
-	cout << "on ";
-
-	for (int i = 0; i < NUM_COLOR; ++i) {
-		if (cfg.cfgAttrPtr->colorBackground == colorMap[i].value) {
-			cout << colorMap[i].name;
-			break;
-		}
-	}
-	cout << '\n';
-}
-
-void	PrintColor(const char *cmd, const ConfigValue &cfg)
-{
-	cout << cmd << ": ";
-	for (int i = 0; i < NUM_COLOR; ++i) {
-		if (*cfg.cfgIntPtr == colorMap[i].value) {
-			cout << colorMap[i].name << ' ';
-			break;
-		}
-	}
-	cout << '\n';
-}
-
 void	PrintBWAttr(const char *cmd, const ConfigValue &cfg)
 {
 	cout << cmd << ": ";
-	if (cfg.cfgAttrPtr->bwAttr) {
+	if ((allConfig[configProfile].*cfg.cfgAttrPtr).bwAttr) {
 		for (int i = 1; i < NUM_ATTR; ++i) {
-			if (cfg.cfgAttrPtr->bwAttr & attrMap[i].value)
+			if ((allConfig[configProfile].*cfg.cfgAttrPtr).bwAttr & attrMap[i].value)
 				cout << attrMap[i].name << ' ';
 		}
 	}
@@ -756,52 +858,52 @@ ConfigFunc cfgFuncBWAttr = { ConfigBWAttr, PrintBWAttr };
 
 #define NUM_CONFIG_INFO	39
 ConfigInfo configInfo[NUM_CONFIG_INFO] = {
-	{ "QuietFullScan", &cfgFuncYesNo, { 0, &kcdConfig.cfgQuietFull, 0, 0 } },
-	{ "QuietSmartScan", &cfgFuncYesNo, { 0, &kcdConfig.cfgQuietSmart, 0, 0 } },
-	{ "QuietPartialScan", &cfgFuncYesNo, { 0, &kcdConfig.cfgQuietPartial, 0, 0 } },
-	{ "AutoScan", &cfgFuncYesNo, { 0, &kcdConfig.cfgAutoScan, 0, 0 } },
-	{ "SpaceSelect", &cfgFuncYesNo, { 0, &kcdConfig.cfgSpaceSelect, 0, 0 } },
-	{ "SortTree", &cfgFuncYesNo, { 0, &kcdConfig.cfgSortTree, 0, 0 } },
-	{ "CaseSensitiveSort", &cfgFuncYesNo, { 0, &kcdConfig.cfgCaseSensitiveSort, 0, 0 } },
-	{ "GraphicChar", &cfgFuncYesNo, { 0, &kcdConfig.cfgGraphicChar, 0, 0 } },
-	{ "ScrollBar", &cfgFuncYesNo, { 0, &kcdConfig.cfgScrollBar, 0, 0 } },
-	{ "GlobDot", &cfgFuncYesNo, { 0, &kcdConfig.cfgGlobDot, 0, 0 } },
-	{ "GlobPath", &cfgFuncYesNo, { 0, &kcdConfig.cfgGlobPath, 0, 0 } },
+	{ "QuietFullScan", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgQuietFull, 0, 0 } },
+	{ "QuietSmartScan", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgQuietSmart, 0, 0 } },
+	{ "QuietPartialScan", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgQuietPartial, 0, 0 } },
+	{ "AutoScan", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgAutoScan, 0, 0 } },
+	{ "SpaceSelect", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgSpaceSelect, 0, 0 } },
+	{ "SortTree", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgSortTree, 0, 0 } },
+	{ "CaseSensitiveSort", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgCaseSensitiveSort, 0, 0 } },
+	{ "GraphicChar", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgGraphicChar, 0, 0 } },
+	{ "ScrollBar", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgScrollBar, 0, 0 } },
+	{ "GlobDot", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgGlobDot, 0, 0 } },
+	{ "GlobPath", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgGlobPath, 0, 0 } },
 
-	{ "ShowListThreshold", &cfgFuncUnsigned, { &kcdConfig.cfgShowListThreshold, 0, 0, 0 } },
+	{ "ShowListThreshold", &cfgFuncUnsigned, { &KcdConfig::cfgShowListThreshold, 0, 0, 0, 0 } },
 
-	{ "StartDir", &cfgFuncStringList, { 0, 0, &kcdConfig.cfgStartDir, 0 } },
-	{ "SkipDir", &cfgFuncStringList, { 0, 0, &kcdConfig.cfgSkipDir, 0 } },
+	{ "StartDir", &cfgFuncStringList, { 0, 0, 0, &KcdConfig::cfgStartDir, 0 } },
+	{ "SkipDir", &cfgFuncStringList, { 0, 0, 0, &KcdConfig::cfgSkipDir, 0 } },
 
-	{ "ClearStartDir", &cfgFuncClearStringList, { 0, 0, &kcdConfig.cfgStartDir, 0 } },
-	{ "ClearSkipDir", &cfgFuncClearStringList, { 0, 0, &kcdConfig.cfgSkipDir, 0 } },
+	{ "ClearStartDir", &cfgFuncClearStringList, { 0, 0, 0, &KcdConfig::cfgStartDir, 0 } },
+	{ "ClearSkipDir", &cfgFuncClearStringList, { 0, 0, 0, &KcdConfig::cfgSkipDir, 0 } },
 
-	{ "NormalColorAttr", &cfgFuncColorAttr, { 0, 0, 0, &kcdConfig.cfgAttrNormal } },
-	{ "NormalBWAttr", &cfgFuncBWAttr, { 0, 0, 0, &kcdConfig.cfgAttrNormal } },
-	{ "DirColorAttr", &cfgFuncColorAttr, { 0, 0, 0, &kcdConfig.cfgAttrLink } },
-	{ "DirBWAttr", &cfgFuncBWAttr, { 0, 0, 0, &kcdConfig.cfgAttrLink } },
-	{ "HighlightDirColorAttr", &cfgFuncColorAttr, { 0, 0, 0, &kcdConfig.cfgAttrHighlight } },
-	{ "HighlightDirBWAttr", &cfgFuncBWAttr, { 0, 0, 0, &kcdConfig.cfgAttrHighlight } },
-	{ "SymLinkColorAttr", &cfgFuncColorAttr, { 0, 0, 0, &kcdConfig.cfgAttrLinkItalic } },
-	{ "SymLinkBWAttr", &cfgFuncBWAttr, { 0, 0, 0, &kcdConfig.cfgAttrLinkItalic } },
-	{ "HighlightSymLinkColorAttr", &cfgFuncColorAttr, { 0, 0, 0, &kcdConfig.cfgAttrHighlightItalic } },
-	{ "HighlightSymLinkAttr", &cfgFuncBWAttr, { 0, 0, 0, &kcdConfig.cfgAttrHighlightItalic } },
+	{ "NormalColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrNormal } },
+	{ "NormalBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrNormal } },
+	{ "DirColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrLink } },
+	{ "DirBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrLink } },
+	{ "HighlightDirColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrHighlight } },
+	{ "HighlightDirBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrHighlight } },
+	{ "SymLinkColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrLinkItalic } },
+	{ "SymLinkBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrLinkItalic } },
+	{ "HighlightSymLinkColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrHighlightItalic } },
+	{ "HighlightSymLinkAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrHighlightItalic } },
 
-	{ "TitleColorAttr", &cfgFuncColorAttr, { 0, 0, 0, &kcdConfig.cfgAttrTitle } },
-	{ "TitleBWAttr", &cfgFuncBWAttr, { 0, 0, 0, &kcdConfig.cfgAttrTitle } },
-	{ "StatusColorAttr", &cfgFuncColorAttr, { 0, 0, 0, &kcdConfig.cfgAttrURL } },
-	{ "StatusBWAttr", &cfgFuncBWAttr, { 0, 0, 0, &kcdConfig.cfgAttrURL } },
-	{ "MoreColorAttr", &cfgFuncColorAttr, { 0, 0, 0, &kcdConfig.cfgAttrMore } },
-	{ "MoreBWAttr", &cfgFuncBWAttr, { 0, 0, 0, &kcdConfig.cfgAttrMore } },
+	{ "TitleColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrTitle } },
+	{ "TitleBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrTitle } },
+	{ "StatusColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrURL } },
+	{ "StatusBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrURL } },
+	{ "MoreColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrMore } },
+	{ "MoreBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrMore } },
 
-	{ "ScrollArrowColorAttr", &cfgFuncColorAttr, { 0, 0, 0, &kcdConfig.cfgAttrScrollArrow } },
-	{ "ScrollArrowBWAttr", &cfgFuncBWAttr, { 0, 0, 0, &kcdConfig.cfgAttrScrollArrow } },
-	{ "ScrollBlockColorAttr", &cfgFuncColorAttr, { 0, 0, 0, &kcdConfig.cfgAttrScrollBlock } },
-	{ "ScrollBlockBWAttr", &cfgFuncBWAttr, { 0, 0, 0, &kcdConfig.cfgAttrScrollBlock } },
-	{ "ScrollBarColorAttr", &cfgFuncColorAttr, { 0, 0, 0, &kcdConfig.cfgAttrScrollBar } },
-	{ "ScrollBarBWAttr", &cfgFuncBWAttr, { 0, 0, 0, &kcdConfig.cfgAttrScrollBar } },
+	{ "ScrollArrowColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollArrow } },
+	{ "ScrollArrowBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollArrow } },
+	{ "ScrollBlockColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollBlock } },
+	{ "ScrollBlockBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollBlock } },
+	{ "ScrollBarColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollBar } },
+	{ "ScrollBarBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollBar } },
 
-	{ "DefaultBackground", &cfgFuncColor, { &kcdConfig.cfgDefaultBackground, 0, 0, 0 } }
+	{ "DefaultBackground", &cfgFuncColor, { 0, &KcdConfig::cfgDefaultBackground, 0, 0, 0 } }
 };
 
 /*************************************************************************
@@ -832,6 +934,8 @@ void	LoadConfig(const string &confFile, int quiet, bool checkSecure = false)
 	if (file == NULL)			// File not found
 		return;
 
+	SetConfigProfile("*");			// Default profile
+
 	try {
 		if (checkSecure) {
 			struct stat	stat1, stat2;
@@ -876,14 +980,52 @@ void	LoadConfig(const string &confFile, int quiet, bool checkSecure = false)
 			configLen = strlen(configLine);
 		
 			if (configLen > 0) {	// Non empty/non comment line
-			
 				size_t i = SeekNonSpace(configLine, 0, configLen);
 				if (i == configLen)	// Blank line
 					continue;	// Check for EOF and 
 							// read another line
-					
-				int	k;
 
+							// Profile
+				if (configLine[i] == '[') {
+					size_t j = 0;
+					try {
+						j = SeekChar(']', configLine, i+1, configLen, 2);
+					}
+					catch (...) {
+						cerr << progName
+						     << _(": missing `]\'\n");
+						PrintErrorCommand(configLine);
+						numError++;
+						continue;
+					}
+
+					string s(configLine+i+1, j-i-1);
+
+					if (!IsEmpty(configLine, j+1, configLen)) {
+						cerr << progName
+						     << _(": extra characters after profile name\n");
+						PrintErrorCommand(configLine);
+						numError++;
+						continue;
+					}
+
+					if (!IsProfileValid(s, true)) {
+						cerr << progName
+						     << _(": invalid charactors in profile name\n");
+						PrintErrorCommand(configLine);
+						numError++;
+						continue;
+					}
+					SetConfigProfile(s);
+
+							// Initialize from [*] if not exist
+					if (!IsConfigProfileGlobal() && allConfig.find(s) == allConfig.end())
+						allConfig[s] = allConfig["*"];
+
+					continue;
+				}
+
+				int	k;
 				for (k = 0; k < NUM_CONFIG_INFO; k++) {
 					const char	*cfgName = configInfo[k].cfgName;
 					size_t	cfgNameLen = strlen(cfgName);
@@ -920,10 +1062,6 @@ void	LoadConfig(const string &confFile, int quiet, bool checkSecure = false)
 
 	fclose(file);
 	
-			// Transfer all config to relevant variables
-	isACSFallBack = !kcdConfig.cfgGraphicChar;
-	khGeometryManager::disableScrollBar = !kcdConfig.cfgScrollBar;
-
 	if (numError > 0) {
 		cerr << _("Error found, press Enter key to continue...\n");
 		while (getchar() != '\n')
@@ -940,16 +1078,36 @@ void	LoadAllConfig(int quiet)
 	
 	loaded = 1;
 
+	allConfig["*"] = defConfig;
 	LoadConfig(KCD_ETCDIR "/kcd.conf", quiet);
 	LoadConfig(confLocalFile, quiet, checkSecureUserConfig);
+
+					// Initialize from [*] if not exist
+	if (allConfig.find(GetCurrentProfile()) == allConfig.end()) {
+		kcdConfig = allConfig["*"];
+		allConfig[GetCurrentProfile()] = kcdConfig;
+	}
+	else
+		kcdConfig = allConfig[GetCurrentProfile()];
+
+			// Transfer all config to relevant variables
+	isACSFallBack = !kcdConfig.cfgGraphicChar;
+	khGeometryManager::disableScrollBar = !kcdConfig.cfgScrollBar;
 }
 
 /*************************************************************************
 	Display configuration function.
 *************************************************************************/
 
+void	PrintProfile()
+{
+	gtout(cout, _("Profile: %$\n")) << GetCurrentProfile();
+}
+
 void	PrintConfig()
 {
+	PrintProfile();
+	SetConfigProfile(GetCurrentProfile());
 	for (int i = 0; i < NUM_CONFIG_INFO; ++i) {
 		if (configInfo[i].cfgFunc->cfgPrintFunc && !configInfo[i].cfgValue.cfgAttrPtr
 		    && configInfo[i].cfgFunc->cfgPrintFunc != PrintColor)
@@ -960,6 +1118,8 @@ void	PrintConfig()
 
 void	PrintAttrConfig()
 {
+	PrintProfile();
+	SetConfigProfile(GetCurrentProfile());
 	for (int i = 0; i < NUM_CONFIG_INFO; ++i) {
 		if (configInfo[i].cfgFunc->cfgPrintFunc && 
 		    (configInfo[i].cfgValue.cfgAttrPtr || configInfo[i].cfgFunc->cfgPrintFunc == PrintColor))
