@@ -822,7 +822,7 @@ FindDirInfo *FindDir(const string &str, bool honor_cfg, bool change)
 void	GetBookmarks(list<MatchInfo> &match_list, const string &linktext);
 
 void	DirListCD(list<MatchInfo> &match_list, const string &init_pos,
-		  const string &str)
+		  const string &str, size_t adjust)
 {
 	list<MatchInfo>::iterator iter;
 	list<MatchInfo>::iterator current;
@@ -840,9 +840,12 @@ void	DirListCD(list<MatchInfo> &match_list, const string &init_pos,
 	}
 	else {
 		iter = current;
-		next_loop(match_list, iter);
+		next_loop(match_list, iter);	// Start from the next one
 	}
 
+						// Loop until we find a
+						// directory that actually
+						// exists
 	while (k_access((*iter).name, F_OK)) {
 		cout << flush;			// Avoid out-of-order display
 		gtout(cerr, _("%$: skipping %$"
@@ -868,6 +871,9 @@ void	DirListCD(list<MatchInfo> &match_list, const string &init_pos,
 						// Output desired dir
 	k_fputs((*iter).name, file);
 	fclose(file);
+
+	if (match_list.size() > adjust+1)	// More than one matched dir
+	cout << _("change to ") << (*iter).name << '\n';
 }
 
 void	CD(const string &linktext, bool entire_tree)
@@ -920,7 +926,7 @@ void	CD(const string &linktext, bool entire_tree)
 			}
 		}
 
-		int	adjust = 0;			// In case getcwd fails
+		size_t	adjust = 0;			// In case getcwd fails
 		for (list<MatchInfo>::iterator iter = match_list.begin();
 		     iter != match_list.end(); ++iter) {
 			if ((*iter).is_curdir) {
@@ -945,7 +951,7 @@ void	CD(const string &linktext, bool entire_tree)
 			
 		}
 		else {					// Jump immediately
-			DirListCD(match_list, saveCwd, linktext);
+			DirListCD(match_list, saveCwd, linktext, adjust);
 			return;
 		}
 	}
@@ -1038,12 +1044,20 @@ void	AddBookmark(const string &str, bool verify = true)
 	string s = ExpandDirectory(str, false);
 	if (verify) {
 		FindDirInfo *f = FindDir(s, true, false);
-		if (f)
-			bookmark_list.push_back(s);
+		if (f) {
+							// Prevent duplicate entries
+			if (find(bookmark_list.begin(), bookmark_list.end(), s)
+			    == bookmark_list.end())
+				bookmark_list.push_back(s);
+			else {
+				cout << flush;		// Avoid out-of-order display
+				gtout(cerr, _("%$: directory %$ already bookmarked\n"))
+				     << progName << s;
+			}
+		}
 		else {
 			cout << flush;			// Avoid out-of-order display
-			gtout(cerr, _("%$: cannot find %$ in saved directory tree\n"
-				      "        Directory ignored\n"))
+			gtout(cerr, _("%$: cannot find %$ in saved directory tree\n"))
 			     << progName << s;
 		}
 		delete f;
@@ -1074,7 +1088,16 @@ void	GetBookmarks(list<MatchInfo> &match_list, const string &linktext)
 	     iter != bookmark_list.end(); iter++) {
 
 		if (linktext.size()) {
-			if (iter->find(linktext) != string::npos) {
+							// Search string after the
+							// last '/'
+			size_t slash_pos = iter->rfind('/');
+			size_t link_pos;
+			if (slash_pos != string::npos)
+				link_pos = iter->find(linktext, slash_pos+1);
+			else
+				link_pos = iter->find(linktext);
+
+			if (link_pos != string::npos) {
 				FindDirInfo *f = FindDir(*iter, true, false);
 				if (f)
 					match_list.push_back(MatchInfo(*iter, f->dir_ptr, false));
@@ -1105,7 +1128,7 @@ void	OutputBookmarks(const string &filename)
 
 	for (list<string>::const_iterator iter = bookmark_list.begin();
 	     iter != bookmark_list.end(); iter++) {
-		ofs << *iter << '\n';
+		ofs << QuoteShellChars(*iter) << '\n';
 		if (!ofs)
 			throw ErrorGenericFile(_("error writing file %$"), s);
 	}
@@ -1137,7 +1160,11 @@ void	ImportBookmarks(const string &filename)
 			break;
 		if (!ifs)
 			throw ErrorGenericFile(_("error reading file %$"), s);
-		AddBookmark(buffer, false);
+		try {
+			AddBookmark(UnquoteShellChars(buffer), false);
+		}
+		catch (ErrorRange &) {
+		}
 	}
 }
 
