@@ -1,6 +1,6 @@
 /*
 	gentree.h	Generate Directory Tree For Display Header
-	Copyright (c) 2000 Kriang Lerdsuwanakij
+	Copyright (c) 2000, 2002, 2003 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -29,10 +29,10 @@ extern string saveCwd;
 string	my_strftime(const char *format, const struct tm *tm);
 
 struct FindDirInfo {
-	DirectoryEntry *dir_ptr;
+	sptr<DirectoryEntry> dir_ptr;
 	string		str;
 
-	FindDirInfo(DirectoryEntry *dir_ptr_, const string &str_) : 
+	FindDirInfo(sptr<DirectoryEntry> dir_ptr_, const string &str_) : 
 		dir_ptr(dir_ptr_), str(str_)
 	{
 	}
@@ -47,7 +47,7 @@ void	ShowBookmarks(const string &linktext);
 void	AddBookmarks(int argc, char *argv[], int argcBegin);
 void	RemoveBookmarks(int argc, char *argv[], int argcBegin);
 
-void	ImportBookmarks(const string &file, bool load);
-void	ExportBookmarks(const string &file, bool load);
+void	ImportBookmarks(const string &file);
+void	ExportBookmarks(const string &file);
 
 #endif	/* __K_GENTREE_H */
