@@ -142,7 +142,7 @@ size_t	GetSourceLine(char *buffer, size_t index)
 
 void	ThrowErrorMessage(const string &msg, char *buffer, size_t from, size_t length)
 {
-	ostrstream bufstr;
+	gtstream bufstr;
 	gtout(bufstr, msg) << GetSourceLine(buffer, from);
 	bufstr << _("Content:\n")) << dump_bytes(buffer, from, length, 70);
 	throw ErrorGenericSyntax(bufstr);
@@ -150,7 +150,7 @@ void	ThrowErrorMessage(const string &msg, char *buffer, size_t from, size_t leng
 
 void	ThrowErrorTag1(const string &tag, const string &msg, char *buffer, size_t from, size_t length)
 {
-	ostrstream bufstr;
+	gtstream bufstr;
 	gtout(bufstr, msg) << tag << GetSourceLine(buffer, from);
 	bufstr << _("Content:\n")) << dump_bytes(buffer, from, length, 70);
 	throw ErrorGenericSyntax(bufstr);
@@ -158,7 +158,7 @@ void	ThrowErrorTag1(const string &tag, const string &msg, char *buffer, size_t f
 
 void	ThrowErrorTag2(const string &tag1, const string &tag2, const string &msg, char *buffer, size_t from, size_t length)
 {
-	ostrstream bufstr;
+	gtstream bufstr;
 	gtout(bufstr, msg) << tag1 << tag2 << GetSourceLine(buffer, from);
 	bufstr << _("Content:\n")) << dump_bytes(buffer, from, length, 70);
 	throw ErrorGenericSyntax(bufstr);
