@@ -252,9 +252,8 @@ int	khScreenManager::ReadKeyboard()
 				RequestRefresh();
 			else if (c == 6) {		// ^F
 				isCtrlF = !isCtrlF;	// Toggle state
-
-// FIXME - Display ^F status here
-
+				if (ctrlWin)
+					ctrlWin->ProcessCtrl('F', isCtrlF);
 			}
 			else if (isCtrlF) {
 				isCtrlF = false;
@@ -1895,8 +1894,23 @@ void	khHyperWindowWithFind::FindProcessKey(int ch)
 
 void	khHyperWindowWithFind::ProcessKey(int ch)
 {
+	bool	start_find = false;
 				// Non-special keys
-	if (ch <= 0xFF && isalnum(ch)) {
+#ifdef USE_UTF8_MODE
+	if (IsUTF8Mode()) {
+		if ((ch <= 0x7F && isalnum(ch)) || (ch > 0x7F && ch <= 0xFF))
+			start_find = true;
+	}
+	else {
+		if (ch <= 0xFF && isalnum(ch))
+			start_find = true;
+	}
+#else
+	if (ch <= 0xFF && isalnum(ch))
+		start_find = true;
+#endif
+
+	if (start_find) {
 				// Start find
 
 				// Pass control of keyboard & cursor to
