@@ -278,7 +278,11 @@ void	FullScanDir(ScanLog &scan_log)
 		for (DirList::iterator iter = kcdConfig.cfgStartDir->begin();
 		     iter != kcdConfig.cfgStartDir->end(); ++iter) {
 
-			FullScanFromDir(scan_log, UnquoteShellChars((*iter)->dir));
+			try {
+				FullScanFromDir(scan_log, UnquoteShellChars((*iter)->dir));
+			}
+			catch (ErrorRange &) {		// Ignore
+			}
 		}
 		
 		if (kcdConfig.cfgSortTree)
@@ -569,27 +573,32 @@ void	SmartScanDir(ScanLog &scan_log)
 	else {
 		for (DirList::iterator iter = kcdConfig.cfgStartDir->begin();
 		     iter != kcdConfig.cfgStartDir->end(); ++iter) {
-			bool found_dir = false;
-			string	dir_name = UnquoteShellChars((*iter)->dir);
 
-			sptr<DirectoryEntry> d2(new DirectoryEntry(dir_name));
+			try {
+				bool found_dir = false;
+				string	dir_name = UnquoteShellChars((*iter)->dir);
 
-			for (sptr_list<DirectoryEntry>::iterator iter2 = dirTree.begin();
-			     iter2 != dirTree.end(); ++iter2) {
-				if ((*iter2)->GetNameStr() == dir_name
-				    && found_dir == false
-				    && d2->IsDir()
-				    && (*iter2)->IsSymLink() == d2->IsSymLink()) {
-						// This is the directory to keep
-					(*iter2)->flags = 2;
-					found_dir = true;
+				sptr<DirectoryEntry> d2(new DirectoryEntry(dir_name));
+
+				for (sptr_list<DirectoryEntry>::iterator iter2 = dirTree.begin();
+				     iter2 != dirTree.end(); ++iter2) {
+					if ((*iter2)->GetNameStr() == dir_name
+					    && found_dir == false
+					    && d2->IsDir()
+					    && (*iter2)->IsSymLink() == d2->IsSymLink()) {
+							// This is the directory to keep
+						(*iter2)->flags = 2;
+						found_dir = true;
+					}
+				}
+
+					// If it wasn't in previous StartDir
+					// start full scan
+				if (found_dir == false) {
+					FullScanFromDir(scan_log, dir_name, true);
 				}
 			}
-
-				// If it wasn't in previous StartDir
-				// start full scan
-			if (found_dir == false) {
-				FullScanFromDir(scan_log, dir_name, true);
+			catch (ErrorRange &) {		// Ignore
 			}
 		}
 	}
