@@ -103,6 +103,42 @@ char	charNeedShellQuote[] = "\'\"`\\";
 char	charNoNeedURLQuote[] = "/$-_@.&+-";	// Slash + safe chars
 char	charHex[] = "0123456789abcdef";
 
+int	HexToBin(char hex)
+{
+	if (hex >= '0' && hex <= '9')
+		return hex-'0';
+	if (hex >= 'a' && hex <= 'f')
+		return hex-'a'+10;
+	if (hex >= 'A' && hex <= 'F')
+		return hex-'A'+10;
+	throw ErrorRange();
+}
+
+string	QuoteShellChars(const string &src)	// Remove quotes
+{
+	string::size_type	len = src.size();
+	string	dest;
+	bool	quoted = false;
+	for (string::size_type i = 0; i < len; i++) {
+		unsigned char c = static_cast<unsigned char>(src[i]);
+		if (strchr(charNeedShellQuote, c)) {
+			dest += '\\';
+			dest += c;
+		}
+		else if (isgraph(c) || c >= 0x80)	// Printable (except
+							// space) and UTF-8
+							// chars
+			dest += c;
+		else {
+			dest += '\\';
+			dest += 'x';
+			dest += charHex[c/16];
+			dest += charHex[c%16];
+		}
+	}
+	return dest;
+}
+
 string	UnquoteShellChars(const string &src)	// Remove quotes
 {
 	string::size_type	len = src.size();
@@ -110,7 +146,17 @@ string	UnquoteShellChars(const string &src)	// Remove quotes
 	bool	quoted = false;
 	for (string::size_type i = 0; i < len; i++) {
 		if (quoted) {
-			dest += src[i];
+			switch (src[i]) {
+				case 'x':
+					if (len < i+3)
+						throw ErrorRange();
+					dest += HexToBin(static_cast<unsigned char>(src[i+1]))*16
+							   +HexToBin(static_cast<unsigned char>(src[i+2]));
+					i += 2;
+					break;
+				default:
+					dest += src[i];
+			}
 			quoted = false;
 		}
 		else if (src[i] == '\\') {
@@ -140,13 +186,6 @@ string 	QuoteURLChars(const string &src)	// Quotes special symbols
 	return dest;
 }
 
-int	HexToBin(char hex)
-{
-	if (hex >= '0' && hex <= '9')
-		return hex-'0';
-	return tolower(hex)-'a'+10;
-}
-
 string	UnquoteURLChars(const string &src)	// Remove quotes
 {
 	string::size_type	len = src.size();
